/* Voxmaps of byte-valued voxels.
 */
#ifndef BYTE_VOXMAP_H
#define BYTE_VOXMAP_H

#ifdef __cplusplus
extern "C" {
#endif

#include "DoubleVoxmap.h"
#include "byte.h"

typedef struct {
  byte ***_voxels;
  ulong width;
  ulong height;
  ulong depth;
  ulong numVoxels;
} ByteVoxmap;

/* Create a new voxmap with each voxel set to 0.
 */
void byte_voxmap_new(
    ByteVoxmap **p_this,
    ulong width,
    ulong height,
    ulong depth);

/* Create a new ByteVoxmap from a DoubleVoxmap.
 */
void byte_voxmap_new_from_double(ByteVoxmap **p_this, DoubleVoxmap *other);

/* Create a new ByteVoxmap from a DoubleVoxmap diff, encoding as per
 * JPEG-Clear.
 */
void byte_voxmap_new_from_double_diff(ByteVoxmap **p_this, DoubleVoxmap *diff);

/* Create a new ByteVoxmap from the absolute values of a DoubleVoxmap diff,
 * which doesn't require any special encoding.
 */
void byte_voxmap_new_from_abs_double_diff(
    ByteVoxmap **p_this,
    DoubleVoxmap *diff);

/* Destructor.
 */
void byte_voxmap_delete(ByteVoxmap **p_this);

/* Print.
 */
void byte_voxmap_print(ByteVoxmap *this, Printer *p);

/* Create a new DoubleVoxmap from a ByteVoxmap.
 */
void double_voxmap_new_from_byte(DoubleVoxmap **p_this, ByteVoxmap *other);

/* Create a new DoubleVoxmap from a ByteVoxmap encoded as per JPEG-Clear.
 */
void double_voxmap_new_from_byte_diff(
    DoubleVoxmap **p_this,
    ByteVoxmap *other);

/* Accessors. Can also access directly from the voxels member if desired.
 */
byte byte_voxmap_get(ByteVoxmap *this, ulong i, ulong j, ulong k);
void byte_voxmap_set(ByteVoxmap *this, ulong i, ulong j, ulong k, byte b);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _H */
