/* Voxmaps of double-valued voxels.
 */
#ifndef DOUBLE_VOXMAP_H
#define DOUBLE_VOXMAP_H

#ifdef __cplusplus
extern "C" {
#endif

#include "jpc.h"

typedef struct {
  double ***_voxels;
  ulong width;
  ulong height;
  ulong depth;
  ulong numVoxels;
} DoubleVoxmap;

/* Create a new voxmap with each voxel set to 0.
 */
void double_voxmap_new(
    DoubleVoxmap **p_this,
    ulong width,
    ulong height,
    ulong depth);

/* Create a new voxmap by adding one to another.
 */
void double_voxmap_new_add(
    DoubleVoxmap **p_this,
    DoubleVoxmap *a,
    DoubleVoxmap *b);

/* Create a new vovmap by subtracting one from another.
 */
void double_voxmap_new_subtract(
    DoubleVoxmap **p_this,
    DoubleVoxmap *a,
    DoubleVoxmap *b);

/* Destructor.
 */
void double_voxmap_delete(DoubleVoxmap **p_this);

/* Print.
 */
void double_voxmap_print(DoubleVoxmap *this, Printer *p);

/* Get a given voxel.
 */
double double_voxmap_get(DoubleVoxmap *this, ulong i, ulong j, ulong k);

/* Set a given voxel.
 */
void double_voxmap_set(DoubleVoxmap *this, ulong i, ulong j, ulong k, double d);

/* Find the maximum absolute difference between the corresponding voxesl of two
 * voxmaps.
 */
double double_voxmap_max_abs_diff(DoubleVoxmap *a, DoubleVoxmap *b);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _H */
