/* Static library providing methods for applying x, y, and z kernels to a
 * DoubleVoxmap.
 */
#ifndef DOUBLE_VOXMAP_KERNEL_H
#define DOUBLE_VOXMAP_KERNEL_H

#ifdef __cplusplus
extern "C" {
#endif

#include "DoubleVoxmap.h"
#include "Kernel.h"

/* Create a new DoubleVoxmap by applying x, y, and z kernels to an existing
 * DoubleVoxmap.
 */
DoubleVoxmap *double_voxmap_new_apply_xyz_kernels(
    DoubleVoxmap **p_this,
    DoubleVoxmap *other,
    Kernel *x_kernel,
    Kernel *y_kernel,
    Kernel *z_kernel);

/* Create a new DoubleVoxmap by applying multiple sets of x, y, and z kernels
 * to an existing DoubleVoxmap. I.e. x_kernels[0] and y_kernels[0] are applied
 * to 'other', and then x_kernels[1] and y_kernels[1] are applied to the reslt,
 * and so on.
 */
DoubleVoxmap *double_voxmap_new_apply_multiple_xyz_kernels(
    DoubleVoxmap **p_this,
    DoubleVoxmap *other,
    ulong num_kernel_pairs,
    Kernel **x_kernels,
    Kernel **y_kernels,
    Kernel **z_kernels);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _H */
