/* Static library providing methods for applying Magic Kernel Sharp and related
 * kernels to DoubleVoxmaps, as efficiently as possible.
 */
#ifndef MAGIC_DOUBLE_VOXMAP_H
#define MAGIC_DOUBLE_VOXMAP_H

#ifdef __cplusplus
extern "C" {
#endif

#include "double_voxmap_kernel.h"
#include "magic.h"

/* Resize a DoubleVoxmap. The methods available, and the parameters 'a' and 'v,'
 * are described in magic.h.
 */
DoubleVoxmap *double_voxmap_new_resize(
    DoubleVoxmap **p_this,
    DoubleVoxmap *other,
    ulong width,
    ulong height,
    ulong depth,
    double k,
    char *method,
    byte a,
    byte v);

/* Blur a DoubleVoxmap. The "width" (two standard deviations) of the blurring is
 * approximately equal to 'blur'.
 */
DoubleVoxmap *double_voxmap_new_blur(
    DoubleVoxmap **p_this,
    DoubleVoxmap *other,
    double blur);

/* Sharpen a voxmap. A sharpening of 1.0 "undoes" a blur of 1.0 above, to first
 * order.
 */
DoubleVoxmap *double_voxmap_new_sharpen(
    DoubleVoxmap **p_this,
    DoubleVoxmap *other,
    double sharpening);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _H */
