#include "ByteVoxmapTest.h"

START

TEST(ByteVoxmap, New) {
  ByteVoxmap *_bv = null;

  byte_voxmap_new(&_bv, 3, 4, 5);
  EXPECT_ULEQ(3, _bv->width);
  EXPECT_ULEQ(4, _bv->height);
  EXPECT_ULEQ(5, _bv->depth);
  EXPECT_ULEQ(60, _bv->numVoxels);
  byte_voxmap_delete(&_bv);
}

TEST(ByteVoxmap, NewFromDouble) {
  ulong i;
  ulong j;
  ulong k;
  DoubleVoxmap *_dv = null;
  ByteVoxmap *_bv = null;

  double_voxmap_new(&_dv, 2, 3, 4);
  byte_voxmap_new_from_double(&_bv, _dv);
  double_voxmap_delete(&_dv);
  EXPECT_ULEQ(2, _bv->width);
  EXPECT_ULEQ(3, _bv->height);
  EXPECT_ULEQ(4, _bv->depth);
  EXPECT_ULEQ(24, _bv->numVoxels);

  for (i = 0; i < 2; i++) {
    for (j = 0; j < 3; j++) {
      for (k = 0; k < 4; k++) {
        EXPECT_ULEQ(0, byte_voxmap_get(_bv, i, j, k));
      }
    }
  }

  double_voxmap_new_from_byte(&_dv, _bv);
  byte_voxmap_delete(&_bv);

  EXPECT_ULEQ(2, _dv->width);
  EXPECT_ULEQ(3, _dv->height);
  EXPECT_ULEQ(4, _dv->depth);

  for (i = 0; i < 2; i++) {
    for (j = 0; j < 3; j++) {
      for (k = 0; k < 4; k++) {
        EXPECT_DEQ(0, double_voxmap_get(_dv, i, j, k));
      }
    }
  }
  double_voxmap_delete(&_dv);

  double_voxmap_new(&_dv, 2, 3, 4);
  double_voxmap_set(_dv, 0, 1, 2, 0.5);
  double_voxmap_set(_dv, 0, 0, 2, 0.25);
  double_voxmap_set(_dv, 1, 2, 2, 1);
  byte_voxmap_new_from_double(&_bv, _dv);
  double_voxmap_delete(&_dv);
  EXPECT_ULEQ(0, byte_voxmap_get(_bv, 0, 0, 0));
  EXPECT_ULEQ(128, byte_voxmap_get(_bv, 0, 1, 2));
  EXPECT_ULEQ(64, byte_voxmap_get(_bv, 0, 0, 2));
  EXPECT_ULEQ(255, byte_voxmap_get(_bv, 1, 2, 2));
  double_voxmap_new_from_byte(&_dv, _bv);
  byte_voxmap_delete(&_bv);
  EXPECT_DEQ(0, double_voxmap_get(_dv, 0, 0, 0));
  EXPECT_DEQ(128.0 / 255, double_voxmap_get(_dv, 0, 1, 2));
  EXPECT_DEQ(64.0 / 255, double_voxmap_get(_dv, 0, 0, 2));
  EXPECT_DEQ(1, double_voxmap_get(_dv, 1, 2, 2));
  double_voxmap_delete(&_dv);
}

TEST(ByteVoxmap, NewFromDoubleDiff) {
  byte a;
  ulong i;
  ulong j;
  ulong k;
  short ss[] = {-254, -239, -220, -168, -101, -99, -68, -40, -27, -25, -9, -1,
    0, 1, 3, 25, 27, 38, 40, 109, 111, 189, 193, 254};
  ByteVoxmap *_bv = null;
  DoubleVoxmap *_dv_final = null;
  DoubleVoxmap *_dv_orig = null;

  double_voxmap_new(&_dv_orig, 2, 3, 4);

  for (a = 0, i = 0; i < 2; i++) {
    for (j = 0; j < 3; j++) {
      for (k = 0; k < 4; k++, a++) {
        _dv_orig->_voxels[i][j][k] = ss[a] / 255.0;
      }
    }
  }

  byte_voxmap_new_from_double_diff(&_bv, _dv_orig);
  double_voxmap_new_from_byte_diff(&_dv_final, _bv);
  byte_voxmap_delete(&_bv);
  EXPECT_DEQ(0, double_voxmap_max_abs_diff(_dv_final, _dv_orig));
  double_voxmap_delete(&_dv_final);
  double_voxmap_delete(&_dv_orig);
}

TEST(ByteVoxmap, Print) {
  char *_e = null;
  char *_s = null;
  ByteVoxmap *_bv = null;

  byte_voxmap_new(&_bv, 1, 1, 1);
  aprint(&_s, 0, byte_voxmap, _bv);
  EXPECT_STREQ(
      "[ByteVoxmap]\n. Width: 1 voxel\n. Height: 1 voxel\n. Depth: 1 voxel\n"
      ". [Contents]\n. . [Slice 0]\n. . . [Row 0]\n. . . . [0] 0x00\n"
      ". . . [/Row 0]\n. . [/Slice 0]\n. [/Contents]\n[/ByteVoxmap]\n",
      _s);
  string_delete(&_s);
  byte_voxmap_delete(&_bv);

  byte_voxmap_new(&_bv, 2, 3, 4);
  aprint(&_s, 0, byte_voxmap, _bv);
  string_new_f(
      &_e,
      "%s%s",
      "[ByteVoxmap]\n. Width: 2 voxels\n. Height: 3 voxels\n"
      ". Depth: 4 voxels\n. [Contents]\n. . [Slice 0]\n. . . [Row 0]\n"
      ". . . . [0] 0x00\n. . . . [1] 0x00\n. . . [/Row 0]\n. . . [Row 1]\n"
      ". . . . [0] 0x00\n. . . . [1] 0x00\n. . . [/Row 1]\n. . . [Row 2]\n"
      ". . . . [0] 0x00\n. . . . [1] 0x00\n. . . [/Row 2]\n. . [/Slice 0]\n"
      ". . [Slice 1]\n. . . [Row 0]\n. . . . [0] 0x00\n. . . . [1] 0x00\n"
      ". . . [/Row 0]\n. . . [Row 1]\n. . . . [0] 0x00\n. . . . [1] 0x00\n"
      ". . . [/Row 1]\n. . . [Row 2]\n. . . . [0] 0x00\n. . . . [1] 0x00\n"
      ". . . [/Row 2]\n",
      ". . [/Slice 1]\n. . [Slice 2]\n. . . [Row 0]\n. . . . [0] 0x00\n"
      ". . . . [1] 0x00\n. . . [/Row 0]\n. . . [Row 1]\n. . . . [0] 0x00\n"
      ". . . . [1] 0x00\n. . . [/Row 1]\n. . . [Row 2]\n. . . . [0] 0x00\n"
      ". . . . [1] 0x00\n. . . [/Row 2]\n. . [/Slice 2]\n. . [Slice 3]\n"
      ". . . [Row 0]\n. . . . [0] 0x00\n. . . . [1] 0x00\n. . . [/Row 0]\n"
      ". . . [Row 1]\n. . . . [0] 0x00\n. . . . [1] 0x00\n. . . [/Row 1]\n"
      ". . . [Row 2]\n. . . . [0] 0x00\n. . . . [1] 0x00\n. . . [/Row 2]\n"
      ". . [/Slice 3]\n. [/Contents]\n[/ByteVoxmap]\n");
  EXPECT_STREQ(_e, _s);
  string_delete(&_s);
  string_delete(&_e);
  byte_voxmap_delete(&_bv);

  byte_voxmap_new(&_bv, 1, 2, 3);
  byte_voxmap_set(_bv, 0, 0, 0, 0x12);
  byte_voxmap_set(_bv, 0, 1, 2, 0x34);
  byte_voxmap_set(_bv, 0, 0, 2, 0xab);
  byte_voxmap_set(_bv, 0, 1, 1, 0xef);
  aprint(&_s, 0, byte_voxmap, _bv);
  EXPECT_STREQ(
      "[ByteVoxmap]\n. Width: 1 voxel\n. Height: 2 voxels\n. Depth: 3 voxels\n"
      ". [Contents]\n. . [Slice 0]\n. . . [Row 0]\n. . . . [0] 0x12\n"
      ". . . [/Row 0]\n. . . [Row 1]\n. . . . [0] 0x00\n. . . [/Row 1]\n"
      ". . [/Slice 0]\n. . [Slice 1]\n. . . [Row 0]\n. . . . [0] 0x00\n"
      ". . . [/Row 0]\n. . . [Row 1]\n. . . . [0] 0xef\n. . . [/Row 1]\n"
      ". . [/Slice 1]\n. . [Slice 2]\n. . . [Row 0]\n. . . . [0] 0xab\n"
      ". . . [/Row 0]\n. . . [Row 1]\n. . . . [0] 0x34\n. . . [/Row 1]\n"
      ". . [/Slice 2]\n. [/Contents]\n[/ByteVoxmap]\n",
      _s);
  string_delete(&_s);
  byte_voxmap_delete(&_bv);
}

TEST(ByteVoxmap, GetSet) {
  byte e;
  ulong i;
  ulong j;
  ulong k;
  ByteVoxmap *_bv = null;

  byte_voxmap_new(&_bv, 2, 3, 4);
  byte_voxmap_set(_bv, 0, 0, 0, 27);
  byte_voxmap_set(_bv, 1, 2, 3, 98);
  byte_voxmap_set(_bv, 0, 0, 1, 7);
  byte_voxmap_set(_bv, 0, 2, 3, 253);

  for (i = 0; i < 2; i++) {
    for (j = 0; j < 3; j++) {
      for (k = 0; k < 4; k++) {
        if (i == 0 && j == 0 && k == 0) {
          e = 27;
        } else if (i == 1 && j == 2 && k == 3) {
          e = 98;
        } else if (i == 0 && j == 0 && k == 1) {
          e = 7;
        } else if (i == 0 && j == 2 && k == 3) {
          e = 253;
        } else {
          e = 0;
        }

        EXPECT_DEQ(e, byte_voxmap_get(_bv, i, j, k));
      }
    }
  }

  byte_voxmap_delete(&_bv);
}

FINISH
