#include "DoubleVoxmapTestDeath.h"

START_DEATH

TEST(DoubleVoxmap, New) {
  DoubleVoxmap *_ = null;

  EXPECT_DEATH(double_voxmap_new(&_, 0, 1, 1), "Width must be nonzero");
  double_voxmap_delete(&_);

  EXPECT_DEATH(double_voxmap_new(&_, 1, 0, 1), "Height must be nonzero");
  double_voxmap_delete(&_);

  EXPECT_DEATH(double_voxmap_new(&_, 1, 1, 0), "Depth must be nonzero");
  double_voxmap_delete(&_);
}

TEST(DoubleVoxmap, Get) {
  DoubleVoxmap *_ = null;

  double_voxmap_new(&_, 1, 1, 1);
  EXPECT_DEATH(
      double_voxmap_get(_, 1, 0, 0),
      "i (1) must be less than width (1)");
  EXPECT_DEATH(
      double_voxmap_get(_, 0, 1, 0),
      "j (1) must be less than height (1)");
  EXPECT_DEATH(
      double_voxmap_get(_, 0, 0, 1),
      "k (1) must be less than depth (1)");
  double_voxmap_delete(&_);
}

TEST(DoubleVoxmap, Get) {
  DoubleVoxmap *_ = null;

  double_voxmap_new(&_, 1, 1, 1);
  EXPECT_DEATH(
      double_voxmap_set(_, 1, 0, 0, 1),
      "i (1) must be less than width (1)");
  EXPECT_DEATH(
      double_voxmap_set(_, 0, 1, 0, 1),
      "j (1) must be less than height (1)");
  EXPECT_DEATH(
      double_voxmap_set(_, 0, 0, 1, 1),
      "k (1) must be less than depth (1)");
  double_voxmap_delete(&_);
}

TEST(DoubleVoxmap, AddSubtractMaxAbsDiff) {
  DoubleVoxmap *_a = null;
  DoubleVoxmap *_b = null;
  DoubleVoxmap *_c = null;

  double_voxmap_new(&_a, 1, 2, 3);
  double_voxmap_new(&_b, 4, 2, 3);
  EXPECT_DEATH(
      double_voxmap_max_abs_diff(_a, _b),
      "Voxmaps have different widths (1 v 4)");
  EXPECT_DEATH(
      double_voxmap_new_add(&_c, _a, _b),
      "Voxmaps have different widths (1 v 4)");
  EXPECT_DEATH(
      double_voxmap_new_subtract(&_c, _a, _b),
      "Voxmaps have different widths (1 v 4)");
  double_voxmap_delete(&_b);
  double_voxmap_delete(&_a);

  double_voxmap_new(&_a, 1, 2, 3);
  double_voxmap_new(&_b, 1, 4, 3);
  EXPECT_DEATH(
      double_voxmap_max_abs_diff(_a, _b),
      "Voxmaps have different heights (2 v 4)");
  EXPECT_DEATH(
      double_voxmap_new_add(&_c, _a, _b),
      "Voxmaps have different heights (2 v 4)");
  EXPECT_DEATH(
      double_voxmap_new_subtract(&_c, _a, _b),
      "Voxmaps have different heights (2 v 4)");
  double_voxmap_delete(&_b);
  double_voxmap_delete(&_a);

  double_voxmap_new(&_a, 1, 2, 3);
  double_voxmap_new(&_b, 1, 2, 4);
  EXPECT_DEATH(
      double_voxmap_max_abs_diff(_a, _b),
      "Voxmaps have different depths (3 v 4)");
  EXPECT_DEATH(
      double_voxmap_new_add(&_c, _a, _b),
      "Voxmaps have different depths (3 v 4)");
  EXPECT_DEATH(
      double_voxmap_new_subtract(&_c, _a, _b),
      "Voxmaps have different depths (3 v 4)");
  double_voxmap_delete(&_b);
  double_voxmap_delete(&_a);
}

FINISH_DEATH
