#include "XmlFileTestDeath.h"

START_DEATH

TEST(XmlFile, Tokens) {
  File *_f = null;
  XmlFile *_xml = null;

  file_new_memory_copy_string(&_f, "<");
  xml_file_new_wrap_existing(&_xml, _f);
  EXPECT_DEATH(
      xml_file_try_read_token(_xml),
      "File ended in the middle of a tag");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "<tag");
  xml_file_new_wrap_existing(&_xml, _f);
  EXPECT_DEATH(
      xml_file_try_read_token(_xml),
      "File ended in the middle of a tag");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "<tag name=");
  xml_file_new_wrap_existing(&_xml, _f);
  EXPECT_DEATH(
      xml_file_try_read_token(_xml),
      "File ended in the middle of a tag");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "<tag></");
  xml_file_new_wrap_existing(&_xml, _f);
  xml_file_read_token(_xml);
  EXPECT_DEATH(
      xml_file_try_read_token(_xml),
      "File ended in the middle of a tag");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "<tag> </");
  xml_file_new_wrap_existing(&_xml, _f);
  xml_file_read_token(_xml);
  EXPECT_DEATH(
      xml_file_try_read_token(_xml),
      "File ended in the middle of a tag");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "<tag> cat dog </");
  xml_file_new_wrap_existing(&_xml, _f);
  xml_file_read_token(_xml);
  xml_file_read_token(_xml);
  EXPECT_DEATH(
      xml_file_try_read_token(_xml),
      "File ended in the middle of a tag");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "<<cat>>");
  xml_file_new_wrap_existing(&_xml, _f);
  EXPECT_DEATH(
      xml_file_try_read_token(_xml),
      "Nested tags");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "  <tag<>>");
  xml_file_new_wrap_existing(&_xml, _f);
  EXPECT_DEATH(
      xml_file_try_read_token(_xml),
      "Nested tags");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, " >");
  xml_file_new_wrap_existing(&_xml, _f);
  EXPECT_DEATH(
      xml_file_try_read_token(_xml),
      "Token started with '>'");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "<tag> >");
  xml_file_new_wrap_existing(&_xml, _f);
  xml_file_read_token(_xml);
  EXPECT_DEATH(
      xml_file_try_read_token(_xml),
      "Token started with '>'");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "a>b");
  xml_file_new_wrap_existing(&_xml, _f);
  EXPECT_DEATH(
      xml_file_try_read_token(_xml),
      "Got unmatched '>' in content");
  xml_file_delete(&_xml);
  file_delete(&_f);
}

TEST(XmlFile, Tags) {
  File *_f = null;
  XmlFile *_xml = null;

  file_new_memory_copy_string(&_f, "\n");
  xml_file_new_wrap_existing(&_xml, _f);
  EXPECT_DEATH(
      xml_file_read_tag(_xml, "dog"),
      "Could not read token");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "dog");
  xml_file_new_wrap_existing(&_xml, _f);
  EXPECT_DEATH(
      xml_file_read_tag(_xml, "dog"),
      "Not a tag");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "</dog>");
  xml_file_new_wrap_existing(&_xml, _f);
  EXPECT_DEATH(
      xml_file_read_start_tag(_xml, "dog"),
      "Expected tag 'dog' to be a start tag but it was not");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "</dog cat=1>");
  xml_file_new_wrap_existing(&_xml, _f);
  EXPECT_DEATH(
      xml_file_read_start_tag_attributes(_xml, "dog", "cat=1"),
      "Expected tag 'dog' with attributes 'cat=1' to be a start tag "
      "but it was not");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "<dog></cat>");
  xml_file_new_wrap_existing(&_xml, _f);
  xml_file_read_tag(_xml, "dog");
  EXPECT_DEATH(
      xml_file_skip_to_end_tag(_xml, "dog", null, null),
      "Found end tag for 'cat' while skipping to end tag for 'dog'");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "<cat></cat>");
  xml_file_new_wrap_existing(&_xml, _f);
  xml_file_read_tag(_xml, "cat");
  EXPECT_DEATH(
      xml_file_skip_to_element(_xml, "dog", null, null),
      "Could not skip to element 'dog'");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "<cat></cat>");
  xml_file_new_wrap_existing(&_xml, _f);
  EXPECT_DEATH(
      xml_file_read_tag(_xml, "dog"),
      "Tag name 'cat' is not expected ('dog')");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "<dog size=1>");
  xml_file_new_wrap_existing(&_xml, _f);
  EXPECT_DEATH(
      xml_file_read_tag_attributes(_xml, "dog", "size=2"),
      "Attributes string 'size=1' is not expected ('size=2')");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "<cat></cat>");
  xml_file_new_wrap_existing(&_xml, _f);
  xml_file_read_tag(_xml, "cat");
  EXPECT_DEATH(
      xml_file_read_start_tag(_xml, "cat"),
      "Expected tag 'cat' to be a start tag but it was not");
  xml_file_delete(&_xml);
  file_delete(&_f);

  file_new_memory_copy_string(&_f, "<dog></dog size=1>");
  xml_file_new_wrap_existing(&_xml, _f);
  xml_file_read_tag(_xml, "dog");
  EXPECT_DEATH(
      xml_file_read_start_tag_attributes(_xml, "dog", "size=1"),
      "Expected tag 'dog' with attributes 'size=1' to be a start tag "
      "but it was not");
  xml_file_delete(&_xml);
  file_delete(&_f);
}

TEST(XmlFile, Elements) {
  File *_f = null;
  XmlFile *_xml = null;
  XmlFile *_el = null;

  file_new_memory_copy_string(&_f, "dog");
  xml_file_new_wrap_existing(&_xml, _f);
  xml_file_read_token(_xml);
  EXPECT_DEATH(
      xml_file_new_memory_read_current_element(&_el, _xml),
      "Must call after reading a start tag");
  xml_file_delete(&_xml);
  file_delete(&_f);
}

FINISH_DEATH
