#include "paw_test.h"

static double c_(double d);


START

TEST(Paw, Size) {
  EXPECT_ULEQ(2, sizeof(paw));
}

TEST(Paw, SpecialValues) {
  EXPECT_DEQ(0, double_from_paw(paw_zero()));
  EXPECT_DEQ(1, double_from_paw(paw_one()));
  EXPECT_DEQ(0.5, double_from_paw(paw_half()));
  EXPECT_NEAR(1.841071527669059e19, double_from_paw(PAW_MAX), 1e4);
  EXPECT_NEAR(2.168404344971009e-19, double_from_paw(PAW_MIN), 1e-38);
}

TEST(Paw, FromParts) {
  paw p;

  p = paw_from_parts(0, 0, 0);
  EXPECT_DEQ(0, double_from_paw(p));

  p = paw_from_parts(0, 0x3f, 0x0);
  EXPECT_DEQ(1, double_from_paw(p));

  p = paw_from_parts(0, 0x3f, 0x80);
  EXPECT_DEQ(1.5, double_from_paw(p));

  p = paw_from_parts(0, 0x3f, 0xc0);
  EXPECT_DEQ(1.75, double_from_paw(p));

  p = paw_from_parts(1, 0x3f, 0x0);
  EXPECT_DEQ(-1, double_from_paw(p));

  p = paw_from_parts(1, 0x3f, 0x80);
  EXPECT_DEQ(-1.5, double_from_paw(p));

  p = paw_from_parts(1, 0x3f, 0xc0);
  EXPECT_DEQ(-1.75, double_from_paw(p));

  p = paw_from_parts(0, 0x40, 0x0);
  EXPECT_DEQ(2, double_from_paw(p));

  p = paw_from_parts(0, 0x40, 0x80);
  EXPECT_DEQ(3, double_from_paw(p));

  p = paw_from_parts(0, 0x40, 0xc0);
  EXPECT_DEQ(3.5, double_from_paw(p));

  p = paw_from_parts(1, 0x40, 0x0);
  EXPECT_DEQ(-2, double_from_paw(p));

  p = paw_from_parts(1, 0x40, 0x80);
  EXPECT_DEQ(-3, double_from_paw(p));

  p = paw_from_parts(1, 0x40, 0xc0);
  EXPECT_DEQ(-3.5, double_from_paw(p));

  p = paw_from_parts(0, 0x3e, 0x0);
  EXPECT_DEQ(0.5, double_from_paw(p));

  p = paw_from_parts(0, 0x3e, 0x80);
  EXPECT_DEQ(0.75, double_from_paw(p));

  p = paw_from_parts(0, 0x3e, 0xc0);
  EXPECT_DEQ(0.875, double_from_paw(p));

  p = paw_from_parts(1, 0x3e, 0x0);
  EXPECT_DEQ(-0.5, double_from_paw(p));

  p = paw_from_parts(1, 0x3e, 0x80);
  EXPECT_DEQ(-0.75, double_from_paw(p));

  p = paw_from_parts(1, 0x3e, 0xc0);
  EXPECT_DEQ(-0.875, double_from_paw(p));
}

TEST(Paw, ComparisonsRelational) {
  paw p1;
  paw p2;

  EXPECT_PAWEQ(paw_zero(), paw_zero());
  EXPECT_FALSE(paw_gt(paw_zero(), paw_zero()));
  EXPECT_FALSE(paw_lt(paw_zero(), paw_zero()));
  EXPECT_TRUE(paw_ge(paw_zero(), paw_zero()));
  EXPECT_TRUE(paw_le(paw_zero(), paw_zero()));

  p1 = paw_make(12.3);
  p2 = paw_make(-3.45e-12);

  EXPECT_PAWEQ(p1, p1);
  EXPECT_FALSE(paw_gt(p1, p1));
  EXPECT_FALSE(paw_lt(p1, p1));
  EXPECT_TRUE(paw_ge(p1, p1));
  EXPECT_TRUE(paw_le(p1, p1));

  EXPECT_FALSE(paw_equals(p1, p2));
  EXPECT_TRUE(paw_gt(p1, p2));
  EXPECT_FALSE(paw_lt(p1, p2));
  EXPECT_TRUE(paw_ge(p1, p2));
  EXPECT_FALSE(paw_le(p1, p2));

  EXPECT_FALSE(paw_equals(p2, p1));
  EXPECT_FALSE(paw_gt(p2, p1));
  EXPECT_TRUE(paw_lt(p2, p1));
  EXPECT_FALSE(paw_ge(p2, p1));
  EXPECT_TRUE(paw_le(p2, p1));

  EXPECT_PAWEQ(p2, p2);
  EXPECT_FALSE(paw_gt(p2, p2));
  EXPECT_FALSE(paw_lt(p2, p2));
  EXPECT_TRUE(paw_ge(p2, p2));
  EXPECT_TRUE(paw_le(p2, p2));

  EXPECT_PAWEQ(p1, paw_max(p1, p1));
  EXPECT_PAWEQ(p1, paw_min(p1, p1));

  EXPECT_PAWEQ(p1, paw_max(p1, p2));
  EXPECT_PAWEQ(p2, paw_min(p1, p2));

  EXPECT_PAWEQ(p1, paw_max(p2, p1));
  EXPECT_PAWEQ(p2, paw_min(p2, p1));

  EXPECT_PAWEQ(p2, paw_max(p2, p2));
  EXPECT_PAWEQ(p2, paw_min(p2, p2));
}

TEST(Paw, ExactIntegers) {
  EXPECT_DEQ(0, c_(0));

  EXPECT_DEQ(1, c_(1));
  EXPECT_DEQ(-1, c_(-1));

  EXPECT_DEQ(2, c_(2));
  EXPECT_DEQ(-2, c_(-2));

  EXPECT_DEQ(3, c_(3));
  EXPECT_DEQ(-3, c_(-3));

  EXPECT_DEQ(4, c_(4));
  EXPECT_DEQ(-4, c_(-4));

  EXPECT_DEQ(7, c_(7));
  EXPECT_DEQ(-7, c_(-7));
  EXPECT_DEQ(8, c_(8));
  EXPECT_DEQ(-8, c_(-8));
  EXPECT_DEQ(9, c_(9));
  EXPECT_DEQ(-9, c_(-9));

  EXPECT_DEQ(15, c_(15));
  EXPECT_DEQ(-15, c_(-15));
  EXPECT_DEQ(16, c_(16));
  EXPECT_DEQ(-16, c_(-16));
  EXPECT_DEQ(17, c_(17));
  EXPECT_DEQ(-17, c_(-17));

  EXPECT_DEQ(31, c_(31));
  EXPECT_DEQ(-31, c_(-31));
  EXPECT_DEQ(32, c_(32));
  EXPECT_DEQ(-32, c_(-32));
  EXPECT_DEQ(33, c_(33));
  EXPECT_DEQ(-33, c_(-33));

  EXPECT_DEQ(63, c_(63));
  EXPECT_DEQ(-63, c_(-63));
  EXPECT_DEQ(64, c_(64));
  EXPECT_DEQ(-64, c_(-64));
  EXPECT_DEQ(65, c_(65));
  EXPECT_DEQ(-65, c_(-65));

  EXPECT_DEQ(127, c_(127));
  EXPECT_DEQ(-127, c_(-127));
  EXPECT_DEQ(128, c_(128));
  EXPECT_DEQ(-128, c_(-128));
  EXPECT_DEQ(129, c_(129));
  EXPECT_DEQ(-129, c_(-129));

  EXPECT_DEQ(254, c_(254));
  EXPECT_DEQ(-254, c_(-254));
  EXPECT_DEQ(255, c_(255));
  EXPECT_DEQ(-255, c_(-255));
  EXPECT_DEQ(256, c_(256));
  EXPECT_DEQ(-256, c_(-256));
  EXPECT_DEQ(257, c_(257));
  EXPECT_DEQ(-257, c_(-257));
  EXPECT_DEQ(258, c_(258));
  EXPECT_DEQ(-258, c_(-258));

  EXPECT_DEQ(510, c_(510));
  EXPECT_DEQ(-510, c_(-510));
  EXPECT_DEQ(511, c_(511));
  EXPECT_DEQ(-511, c_(-511));
  EXPECT_DEQ(512, c_(512));
  EXPECT_DEQ(-512, c_(-512));
}

TEST(Paw, RoundedIntegers) {
  EXPECT_DEQ(512, c_(513));
  EXPECT_DEQ(-512, c_(-513));
  EXPECT_DEQ(514, c_(514));
  EXPECT_DEQ(-514, c_(-514));
  EXPECT_DEQ(516, c_(515));
  EXPECT_DEQ(-516, c_(-515));
  EXPECT_DEQ(516, c_(516));
  EXPECT_DEQ(-516, c_(-516));
  EXPECT_DEQ(516, c_(517));
  EXPECT_DEQ(-516, c_(-517));
  EXPECT_DEQ(518, c_(518));
  EXPECT_DEQ(-518, c_(-518));
  EXPECT_DEQ(520, c_(519));
  EXPECT_DEQ(-520, c_(-519));
  EXPECT_DEQ(520, c_(520));
  EXPECT_DEQ(-520, c_(-520));
  EXPECT_DEQ(520, c_(521));
  EXPECT_DEQ(-520, c_(-521));
  EXPECT_DEQ(522, c_(522));
  EXPECT_DEQ(-522, c_(-522));
  EXPECT_DEQ(524, c_(523));
  EXPECT_DEQ(-524, c_(-523));

  EXPECT_DEQ(1020, c_(1020));
  EXPECT_DEQ(-1020, c_(-1020));
  EXPECT_DEQ(1020, c_(1021));
  EXPECT_DEQ(-1020, c_(-1021));
  EXPECT_DEQ(1022, c_(1022));
  EXPECT_DEQ(-1022, c_(-1022));
  EXPECT_DEQ(1024, c_(1023));
  EXPECT_DEQ(-1024, c_(-1023));
  EXPECT_DEQ(1024, c_(1024));
  EXPECT_DEQ(-1024, c_(-1024));
  EXPECT_DEQ(1024, c_(1025));
  EXPECT_DEQ(-1024, c_(-1025));
  EXPECT_DEQ(1024, c_(1026));
  EXPECT_DEQ(-1024, c_(-1026));
  EXPECT_DEQ(1028, c_(1027));
  EXPECT_DEQ(-1028, c_(-1027));
  EXPECT_DEQ(1028, c_(1028));
  EXPECT_DEQ(-1028, c_(-1028));
  EXPECT_DEQ(1028, c_(1029));
  EXPECT_DEQ(-1028, c_(-1029));
  EXPECT_DEQ(1032, c_(1030));
  EXPECT_DEQ(-1032, c_(-1030));
  EXPECT_DEQ(1032, c_(1031));
  EXPECT_DEQ(-1032, c_(-1031));
  EXPECT_DEQ(1032, c_(1032));
  EXPECT_DEQ(-1032, c_(-1032));
  EXPECT_DEQ(1032, c_(1033));
  EXPECT_DEQ(-1032, c_(-1033));
  EXPECT_DEQ(1032, c_(1034));
  EXPECT_DEQ(-1032, c_(-1034));
  EXPECT_DEQ(1036, c_(1035));
  EXPECT_DEQ(-1036, c_(-1035));
}

TEST(Paw, MaxOverflow) {
  double d;

  d = double_from_parts(0, 1086, 0xff00000000000);
  EXPECT_DEQ(d, c_(d));
  d = double_from_parts(1, 1086, 0xff00000000000);
  EXPECT_DEQ(d, c_(d));

  EXPECT_TRUE(double_is_positive_infinity(
        c_(double_from_parts(0, 1086, 0xff80000000000))));
  EXPECT_TRUE(double_is_negative_infinity(
        c_(double_from_parts(1, 1086, 0xff80000000000))));
  EXPECT_TRUE(double_is_positive_infinity(
        c_(double_from_parts(0, 1087, 0x0))));
  EXPECT_TRUE(double_is_negative_infinity(
        c_(double_from_parts(1, 1087, 0x0))));
}

TEST(Paw, ExactFractions) {
  EXPECT_DEQ(1.0 / 2, c_(1.0 / 2));
  EXPECT_DEQ(-1.0 / 2, c_(-1.0 / 2));

  EXPECT_DEQ(3.0 / 4, c_(3.0 / 4));
  EXPECT_DEQ(-3.0 / 4, c_(-3.0 / 4));

  EXPECT_DEQ(1.0 / 4, c_(1.0 / 4));
  EXPECT_DEQ(-1.0 / 4, c_(-1.0 / 4));

  EXPECT_DEQ(7.0 / 8, c_(7.0 / 8));
  EXPECT_DEQ(-7.0 / 8, c_(-7.0 / 8));
  EXPECT_DEQ(1.0 / 8, c_(1.0 / 8));
  EXPECT_DEQ(-1.0 / 8, c_(-1.0 / 8));
}

TEST(Paw, MinUnderflow) {
  double d;

  d = double_from_parts(0, 961, 0x0);
  EXPECT_DEQ(d, c_(d));
  d = double_from_parts(1, 1086, 0xff00000000000);
  EXPECT_DEQ(d, c_(d));

  EXPECT_DEQ(0, c_(double_from_parts(0, 960, 0xff7ffffffffff)));
  EXPECT_DEQ(0, c_(double_from_parts(1, 960, 0xff7ffffffffff)));
}

TEST(Paw, Special) {
  EXPECT_DEQ(0, c_(double_subnormal()));
  EXPECT_TRUE(double_is_positive_infinity(c_(double_positive_infinity())));
  EXPECT_TRUE(double_is_negative_infinity(c_(double_negative_infinity())));
  EXPECT_TRUE(double_is_nan(c_(double_nan())));

  EXPECT_TRUE(paw_is_zero(paw_zero()));
  EXPECT_FALSE(paw_is_positive_infinity(paw_zero()));
  EXPECT_FALSE(paw_is_negative_infinity(paw_zero()));
  EXPECT_FALSE(paw_is_nan(paw_zero()));
  EXPECT_FALSE(paw_is_infinite_or_nan(paw_zero()));

  EXPECT_TRUE(paw_is_zero(paw_make(0)));
  EXPECT_FALSE(paw_is_positive_infinity(paw_make(0)));
  EXPECT_FALSE(paw_is_negative_infinity(paw_make(0)));
  EXPECT_FALSE(paw_is_nan(paw_make(0)));
  EXPECT_FALSE(paw_is_infinite_or_nan(paw_make(0)));

  EXPECT_FALSE(paw_is_zero(paw_make(1)));
  EXPECT_FALSE(paw_is_positive_infinity(paw_make(1)));
  EXPECT_FALSE(paw_is_negative_infinity(paw_make(1)));
  EXPECT_FALSE(paw_is_nan(paw_make(1)));
  EXPECT_FALSE(paw_is_infinite_or_nan(paw_make(1)));

  EXPECT_FALSE(paw_is_zero(paw_make(-1)));
  EXPECT_FALSE(paw_is_positive_infinity(paw_make(-1)));
  EXPECT_FALSE(paw_is_negative_infinity(paw_make(-1)));
  EXPECT_FALSE(paw_is_nan(paw_make(-1)));
  EXPECT_FALSE(paw_is_infinite_or_nan(paw_make(-1)));

  EXPECT_FALSE(paw_is_zero(paw_make(double_positive_infinity())));
  EXPECT_TRUE(paw_is_positive_infinity(paw_make(double_positive_infinity())));
  EXPECT_FALSE(paw_is_negative_infinity(paw_make(double_positive_infinity())));
  EXPECT_FALSE(paw_is_nan(paw_make(double_positive_infinity())));
  EXPECT_TRUE(paw_is_infinite_or_nan(paw_make(double_positive_infinity())));

  EXPECT_FALSE(paw_is_zero(paw_make(double_negative_infinity())));
  EXPECT_FALSE(paw_is_positive_infinity(paw_make(double_negative_infinity())));
  EXPECT_TRUE(paw_is_negative_infinity(paw_make(double_negative_infinity())));
  EXPECT_FALSE(paw_is_nan(paw_make(double_negative_infinity())));
  EXPECT_TRUE(paw_is_infinite_or_nan(paw_make(double_negative_infinity())));

  EXPECT_FALSE(paw_is_zero(paw_make(double_nan())));
  EXPECT_FALSE(paw_is_positive_infinity(paw_make(double_nan())));
  EXPECT_FALSE(paw_is_negative_infinity(paw_make(double_nan())));
  EXPECT_TRUE(paw_is_nan(paw_make(double_nan())));
  EXPECT_TRUE(paw_is_infinite_or_nan(paw_make(double_nan())));
}

TEST(Paw, Arithmetic) {
  paw p1;
  paw p2;
  paw p3;

  p1 = paw_make(7.5);
  p2 = paw_make(-3.25);
  p3 = paw_make(-1.5);

  EXPECT_PAWEQ(paw_zero(), paw_negate(paw_zero()));
  EXPECT_PAWEQ(paw_nan(), paw_negate(paw_nan()));
  EXPECT_PAWEQ(paw_make(-7.5), paw_negate(p1));
  EXPECT_PAWEQ(paw_make(3.25), paw_negate(p2));
  EXPECT_PAWEQ(paw_make(1.5), paw_negate(p3));
  EXPECT_PAWEQ(PAW_MAX, paw_negate(paw_negate(PAW_MAX)));
  EXPECT_PAWEQ(PAW_MIN, paw_negate(paw_negate(PAW_MIN)));

  EXPECT_PAWEQ(paw_make(4.25), paw_add(p1, p2));

  EXPECT_PAWEQ(paw_make(10.75), paw_subtract(p1, p2));
  EXPECT_PAWEQ(paw_make(-10.75), paw_subtract(p2, p1));

  EXPECT_PAWEQ(paw_make(2.25), paw_multiply(p3, p3));
  EXPECT_PAWEQ(paw_make(-6.5), paw_multiply(p2, paw_make(2)));
  EXPECT_PAWEQ(p2, paw_multiply(p2, paw_one()));

  EXPECT_PAWEQ(paw_make(-1.625), paw_divide(p2, paw_make(2)));

  EXPECT_PAWEQ(p2, paw_bound(p1, p2, p2));
  EXPECT_PAWEQ(p2, paw_bound(p1, paw_make(-10), p2));
  EXPECT_PAWEQ(p1, paw_bound(p2, p1, p1));
  EXPECT_PAWEQ(p1, paw_bound(p2, p1, paw_make(10)));
}

FINISH


static double c_(double d) {
  return double_from_paw(paw_make(d));
}
